=encoding utf8

=head1 Name

SPVM::Document - SPVM Documentation

=head1 SPVM Language Specification

=over 2

=item * L<Language Specification|SPVM::Document::Language>

=over 2

=item * L<Lexical Tokenization|SPVM::Document::Language::Tokenization>

=item * L<Syntax Parsing|SPVM::Document::Language::SyntaxParsing>

=item * L<Class Definition|SPVM::Document::Language::Class>

=item * L<Types|SPVM::Document::Language::Types>

=item * L<Statements|SPVM::Document::Language::Statements>

=item * L<Operators|SPVM::Document::Language::Operators>

=item * L<Exception Handling|SPVM::Document::Language::ExceptionHandling>

=item * L<Garbage Collection|SPVM::Document::Language::GarbageCollection>

=item * L<System Setting|SPVM::Document::Language::System>

=back

=back

=head1 SPVM Environment Variables

=over 2

=item * L<SPVM Environment Variables|SPVM::Document::EnvironmentVariables>

=back

=head1 SPVM Native Class

=over 2

=item * L<Native Classes|SPVM::Document::NativeClass>

=back

=head2 Native Config

=over 2

=item * L<SPVM::Builder::Config - Native Class Config|SPVM::Builder::Config>

=item * L<SPVM::Builder::Config::Linker - Linker Config|SPVM::Builder::Config::Linker>

=item * L<SPVM::Builder::Config::Base - Base Config|SPVM::Builder::Config::Base>

=item * L<SPVM::Builder::Config::Global - Global Config|SPVM::Builder::Config::Global>

=item * L<SPVM::Builder::Config::Exe - Executable Config|SPVM::Builder::Config::Exe>

=item * L<SPVM::Builder::Config::Exe::MSVC - MSVC Executable Config|SPVM::Builder::Config::Exe::MSVC>

=item * L<SPVM::Builder::Config::DLL - DLL Config|SPVM::Builder::Config::DLL>

=item * L<SPVM::Builder::ConfigBuilder - Config Builder|SPVM::Builder::ConfigBuilder>

=item * L<SPVM::Builder::Config::Util - Base Config|SPVM::Builder::Config::Util>

=back

=head2 SPVM Native APIs

=over 2

=item * L<Native APIs|SPVM::Document::NativeAPI>

=item * L<Runtime Native APIs|SPVM::Document::NativeAPI::Runtime>

=item * L<Basic Type Native APIs|SPVM::Document::NativeAPI::BasicType>

=item * L<Class Variable Native APIs|SPVM::Document::NativeAPI::ClassVariable>

=item * L<Field Native APIs|SPVM::Document::NativeAPI::Field>

=item * L<Method Native APIs|SPVM::Document::NativeAPI::Method>

=item * L<Argument Native APIs|SPVM::Document::NativeAPI::Argument>

=item * L<Type Native APIs|SPVM::Document::NativeAPI::Type>

=item * L<Compiler Native APIs|SPVM::Document::NativeAPI::Compiler>

=item * L<Class File Native APIs|SPVM::Document::NativeAPI::ClassFile>

=item * L<String Buffer Native APIs|SPVM::Document::NativeAPI::StringBuffer>

=item * L<Allocatord Native APIs|SPVM::Document::NativeAPI::Allocator>

=item * L<Mutex Native APIs|SPVM::Document::NativeAPI::Mutex>

=item * L<Internal Native APIs|SPVM::Document::NativeAPI::Internal>

=back

=head2 Building Information

=over 2

=item * L<SPVM::Builder::LibInfo - Library Information|SPVM::Builder::LibInfo>

=item * L<SPVM::Builder::LinkInfo - Link Information|SPVM::Builder::LinkInfo>

=item * L<SPVM::Builder::ObjectFileInfo - Object File Information|SPVM::Builder::ObjectFileInfo>

=item * L<SPVM::Builder::CompileInfo - Compilation Information|SPVM::Builder::CompileInfo>

=item * L<SPVM::Builder::CommandInfo - Command Information|SPVM::Builder::CommandInfo>

=item * L<SPVM::Builder::ScriptInfo - SPVM Script Information|SPVM::Builder::ScriptInfo>

=back

=head2 Builder APIs

=over 2

=item * L<SPVM::Builder::API - Builder APIs|SPVM::Builder::API>

=item * L<SPVM::Builder::Util::API - Builder Utility APIs|SPVM::Builder::Util::API>

=back

=head2 Resource System

=over 2

=item * L<Resources|SPVM::Document::Resource>

=back

=head1 SPVM Exchange APIs

=over 2

=item * L<SPVM Exchange APIs|SPVM::ExchangeAPI>

=back

=head1 Commands

=over 2

=item * L<spvm - Executing SPVM Programs|spvm>

=item * L<spvmcc - Generating Executable Files|spvmcc>

=item * L<spvmdist - Distributing SPVM Modules|spvmdist>

=item * L<spvmdeps - Class Dependencies Analyzer|spvmdeps>

=back

=head1 See Also

=over 2

=item * L<SPVM>

=back

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

=cut
